"{ Package: 'stx:libjava' }"

AbstractJavaTestCase subclass:#JavaInterfaceMethodRefTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Java-Tests-RuntimeConstantPool'
!


!JavaInterfaceMethodRefTests methodsFor:'javaInterfaceMethodRef tests'!

testCorrectInstanceCreation
    | javaClassRef  initString  javaMethodRef |

    initString := 'Ljava/lang/Runnable;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2 in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaMethodRef valueCache isNil).
    self assertTrue: (javaMethodRef name = 'run').
    self assertTrue: (javaMethodRef descriptor = '()V').
    self assertTrue: (javaMethodRef classRef name = 'Ljava/lang/Runnable;').

    "Created: / 08-04-2011 / 14:01:41 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:23 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testCorrectResolving
    | javaClassRef  initString  javaMethodRef  expectedResult  result |

    initString := 'Ljava/lang/Runnable;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2        in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    result := javaMethodRef resolve.
    expectedResult := (Java classForName: 'java.lang.Runnable') 
                methodDictionary at: #'run()V'.
    self assertTrue: (result = expectedResult).

    "Created: / 08-04-2011 / 14:07:57 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:25 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidation
    | javaClassRef  javaMethodRef |

    javaClassRef := self getClassRefFor: 'Ljava/lang/Runnable;'.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2 in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaClassRef isResolved).
    javaMethodRef invalidate.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaClassRef isResolved not).

    "Created: / 08-04-2011 / 14:09:06 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:27 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassNegative
    | javaClassRef  javaMethodRef |

    javaClassRef := self getClassRefFor: 'Ljava/lang/Runnable;'.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2        in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaClassRef isResolved).
    javaMethodRef invalidateForClass: 'Ljava/lang/Object;'.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaClassRef isResolved).

    "Created: / 08-04-2011 / 16:23:06 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:30 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testInvalidationForClassPositive
    | javaClassRef  javaMethodRef |

    javaClassRef := self getClassRefFor: 'Ljava/lang/Runnable;'.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2               in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaMethodRef isResolved).
    self assertTrue: (javaClassRef isResolved).
    javaMethodRef invalidateForClass: 'Ljava/lang/Runnable;'.
    self assertTrue: (javaMethodRef isResolved not).
    self assertTrue: (javaClassRef isResolved not).

    "Created: / 08-04-2011 / 16:23:19 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:32 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testResolving
    | javaClassRef  javaMethodRef |

    javaClassRef := self getClassRefFor: 'Ljava/lang/Runnable;'.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaInterfaceMethodRef2 in: nil
                namedAndTyped: (self getName: 'run' descriptor: '()V')
                inClassIdentifiedByRef: javaClassRef.
    self assertTrue: (javaMethodRef isResolved not).
    javaMethodRef resolve.
    self assertTrue: (javaClassRef isResolved).
    self assertTrue: (javaMethodRef isResolved).

    "Created: / 08-04-2011 / 14:04:01 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:34 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaInterfaceMethodRefTests methodsFor:'permission tests'!

testAccessingPrivateFromOutside
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaMethodRef2 in: nil
                namedAndTyped: (self getName: 'privateMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:44:48 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:39 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPrivateFromOutsideInNonPublic
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaMethodRef2        in: nil
                namedAndTyped: (self getName: 'privateMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:47:44 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:41 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPrivateFromSubclass
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.SubclassOfPublicClass').
    javaMethodRef := JavaMethodRef2               in: nil
                namedAndTyped: (self getName: 'privateMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:49:32 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:43 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingProtectedFromOutside
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaMethodRef2                      in: nil
                namedAndTyped: (self getName: 'protectedMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:44:48 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:45 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingProtectedFromOutsideInNonPublic
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaMethodRef2                             in: nil
                namedAndTyped: (self getName: 'protectedMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:47:54 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:47 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingProtectedFromSubclass
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef 
        owner: (Java classForName: 'stx.libjava.tests.mocks.SubclassOfPublicClass').
    javaMethodRef := JavaMethodRef2 in: nil
                namedAndTyped: (self getName: 'protectedMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException isNil).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:49:23 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:50 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPublic
    | javaClassRef  javaMethodRef  initString |

    self enableMockedExceptionThrowing.
    self shouldnt: 
            [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
            javaClassRef := self getClassRefFor: initString.
            javaClassRef owner: (Java classForName: 'java.lang.Object').
            javaMethodRef := JavaMethodRef2 in: nil
                        namedAndTyped: (self getName: 'publicMethod'
                                descriptor: '()Ljava/lang/String;')
                        inClassIdentifiedByRef: javaClassRef.
            javaMethodRef resolve. ]
        raise: Error.
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:44:13 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:53 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPublicFromOutside
    | javaClassRef  javaMethodRef  initString |

    self enableMockedExceptionThrowing.
    self shouldnt: 
            [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
            javaClassRef := self getClassRefFor: initString.
            javaClassRef owner: (Java classForName: 'java.lang.Object').
            javaMethodRef := JavaMethodRef2        in: nil
                        namedAndTyped: (self getName: 'publicMethod'
                                descriptor: '()Ljava/lang/String;')
                        inClassIdentifiedByRef: javaClassRef.
            javaMethodRef resolve. ]
        raise: Error.
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:44:31 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:55 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPublicFromOutsideInNonPublic
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/NonPublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.Object').
    javaMethodRef := JavaMethodRef2                       in: nil
                namedAndTyped: (self getName: 'publicMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException notNil 
                and: [ throwedException messageText = 'IllegalAccessError' ]).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:48:05 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:57 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
!

testAccessingPublicFromSubclass
    | javaClassRef  javaMethodRef  initString  throwedException |

    self enableMockedExceptionThrowing.
    
    [ initString := 'Lstx/libjava/tests/mocks/PublicClass;'.
    javaClassRef := self getClassRefFor: initString.
    javaClassRef owner: (Java classForName: 'java.lang.SubclassOfPublicClass').
    javaMethodRef := JavaMethodRef2 in: nil
                namedAndTyped: (self getName: 'publicMethod'
                        descriptor: '()Ljava/lang/String;')
                inClassIdentifiedByRef: javaClassRef.
    javaMethodRef resolve. ] on: Error
            do: [:e | throwedException := e ].
    self assertTrue: (throwedException isNil).
    self disableMockedExceptionThrowing.

    "Created: / 13-04-2011 / 14:49:11 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
    "Modified: / 10-05-2011 / 16:22:59 / Marcel Hlopko <hlopkmar@fel.cvut.cz>"
! !

!JavaInterfaceMethodRefTests class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
